#!/usr/bin/php -q
<?php
# Made by Sergej Sorli <sergej.sorli@arnes.si> in 2007

setlocale(LC_ALL, 'sl_SI.UTF-8');//1252 ACP UTF-8

//Parse csv file with host data into Cacti database.
//Header file structre:
//<fqdn>;<name>;<community>;<snmp_ver>;<disable>;<snmp_username>;<snmp_password>;<snmp_port>;<snmp_timeout>;<template_name>

/* do NOT run this script through a web browser */
if (!isset($_SERVER["argv"][0])) {
	die("<br><strong>This script is only meant to run at the command line.</strong>");
}
$no_http_headers = true;
#################################################
#
# Well, usage
#
function display_help() {
	echo "Import Devices SOAP Script 1.0, Copyright 2007 - The Cacti Group\n\n";
	echo "A simple command line SOAP utility to import devices in Cacti\n\n";
	echo "usage: auto_add_devices.php --username=[username] --password=[password]\n";
	echo "--urn=[urn] --source=[source file path] [--debug]\n";
}

#####################################################################
#
# Main
#
$debug = false; //default start

/* process calling arguments */
$parms = $_SERVER["argv"];
array_shift($parms);

	foreach($parms as $parameter) {
		@list($arg, $value) = @explode("=", $parameter);
		
		switch($arg)
		{
			case "--username":
				$username = trim($value);
				break;
			case "--password":
				$password = trim($value);
				break;
			case "--urn":
				$urn = trim($value);
				break;
			case "--source":
				$source = trim($value);
				break;
			case "--debug":
				$debug = true;
				break;
			default:
				display_help();
				exit(0);
		}
	}
	
	if ($username == "" || $password == ""){
		display_help();
		exit(1);
	}
	if ($urn == ""){
		display_help();
		exit(1);
	}
	if (!file_exists($source)){
		display_help();
		echo ("FAILED -- " . $source . " not exist\n"); 
		exit(1);
	}

//#############################################################

//Soap configuration
$soap_calls_retries = 3;
$url_Location = 'http://' . $urn . '/soap_service.php';
$urn_Location = 'urn:' . $urn . '';

$opts = array('location' => $url_Location,
'uri' => 'pc_SOAP_return_soap');
$client = new SOAPClient(null, $opts);
$auth = new SOAPAuth ($username, $password);
$header = new SOAPHeader($urn_Location, 'authenticate_user', $auth);
//#############################################################

//Source & destination files
$bckLOG = $source . "log.csv"; //bacup destination base
$errLOG = $source . "err.csv"; //errors
//#############################################################

//open files
if(!$debug)$errhandle = fopen($errLOG, "a");  //err records
if(!$debug)$bckhandle = fopen($bckLOG, "a");  //backup file

$handle = fopen($source, "r");

$row = 0;
while (($data = fgetcsv($handle, 1000, ";")) !== FALSE) {
	$row++; //$num = count($data);

	$retries = $soap_calls_retries;
	//Configuration fields
	$fqdn = $data[0];
	$description = $data[1];
	$community = $data[2];
	$snmp_ver = $data[3];
	$disable = $data[4];
	$snmp_username = $data[5];
	$snmp_password = $data[6];
	$snmp_port = $data[7];
	$snmp_timeout = $data[8];
	$hostTemplateName = $data[9];

	$avail        = 2;
	$ping_method  = 3;
	$ping_port    = 23;
	$ping_timeout = 5000;
	$ping_retries = 3;
	$notes         = "";
	$snmp_auth_protocol   = "";
	$snmp_priv_passphrase = "";
	$snmp_priv_protocol   = "";
	$snmp_context         = "";
	$max_oids			  = 60;
	
	if($debug){
		
        echo "\n\nDetails:\n";
        echo "Row=" . $row . "\n";
        echo "fqdn=" . $fqdn . "\n";
		echo "description=" . $description . "\n";
		echo "community=" . $community . "\n";
		echo "snmp_ver=" . $snmp_ver . "\n";
		echo "disable=" . $disable . "\n";
		echo "snmp_username=" . $snmp_username . "\n";
		echo "snmp_password=" . $snmp_password . "\n";
		echo "snmp_port=" . $snmp_port . "\n";
		echo "snmp_timeout=" . $snmp_timeout . "\n";
        echo "hostTemplateName=" . $hostTemplateName . "\n";
	}

	do {
    	try {
        	$returnResult = $client->__soapCall('return_ID_get_Host_Template_By_Name', array('hostTemplateName' => $hostTemplateName), array(), array($header));
        	if (isset($returnResult["Errors"]) && !empty($returnResult["Errors"])){
        		printThrowException($returnResult["Errors"]);
        	} else {
        		$template_id = $returnResult["Result"];
        
                if($debug){
		        	echo "template_id=" . $template_id . "\n";
        		}
        						
        		if (!$debug) $returnResult = $client->__soapCall('return_ID_add_Device', array(
        		'template_id' => $template_id,
        		'description' => $description,
        		'fqdn' => $fqdn,
        		'community' => $community,
        		'snmp_ver' => $snmp_ver,
        		'snmp_username' => $snmp_username,
        		'snmp_password' => $snmp_password,
        		'snmp_port' => $snmp_port,
        		'snmp_timeout' => $snmp_timeout,
        		'disable' => $disable,
        		'avail' => $avail,
        		'ping_method' => $ping_method,
        		'ping_port' => $ping_port,
        		'ping_timeout' => $ping_timeout,
        		'ping_retries' => $ping_retries,
        		'notes' => $notes,
        		'snmp_auth_protocol' => $snmp_auth_protocol,
        		'snmp_priv_passphrase' => $snmp_priv_passphrase,
        		'snmp_priv_protocol' => $snmp_priv_protocol,
        		'snmp_context' => $snmp_context,
        		'max_oids' => $max_oids
        		), array(), array($header));
        	}
        	$retries = 0;
    	} catch (Exception $e) {
        	echo ("\n" . $row . ". Message = " . $e->__toString() . "\n");
        	echo "Number of retries:" . $retries . "\n";
        	echo "Details:\n";
        	echo "fqdn=" . $fqdn . "\n";
            echo "description=" . $description . "\n";
            echo "community=" . $community . "\n";
            echo "snmp_ver=" . $snmp_ver . "\n";
            echo "disable=" . $disable . "\n";
            echo "snmp_username=" . $snmp_username . "\n";
            echo "snmp_password=" . $snmp_password . "\n";
            echo "snmp_port=" . $snmp_port . "\n";
            echo "snmp_timeout=" . $snmp_timeout . "\n";
            echo "hostTemplateName=" . $hostTemplateName . "\n";
        	$retries--;
    	    if ($retries <= 0) {
        		$errors = array();        		
        		$catchException["value"] = ""; $catchException["variable"] = ""; $catchException["function"] = "SOAP call"; $catchException["description"] = $e->__toString();
            	array_push($errors, $catchException);
				$returnResult["Errors"] = $errors;
        	}
    	}
	} while ($retries > 0);

	//print error and resoult
	if (!$debug)
	if (isset($returnResult["Errors"]) && !empty($returnResult["Errors"])){
		echo $row . ". Errors\n";
		printThrowException($returnResult["Errors"]);
		if (!$debug) fputcsv($errhandle, $data, ";");
	} else {
		echo $row . ". Host Created: (" . $returnResult["Result"] . ")\n";
		if (!$debug) fputcsv($bckhandle, $data, ";");
	}
}

//close files
fclose($handle);    //close source csv file
if(!$debug)fclose($bckhandle); //backup log
if(!$debug)fclose($errhandle); //error log

//SOAP autentication
class SOAPAuth{
	public $username;
	public $password;

	public function __construct($username, $password){
		$this->username = $username;
		$this->password = $password;
	}
}

function printThrowException($errors){
	foreach($errors as $throwException){
		printf ("Error: " . $throwException["description"] . " - " . $throwException["function"] . "/" . $throwException["variable"] . "=" . $throwException["value"] . "\n");
	}
}

?>
