#!/usr/bin/php -q
<?php
# Made by Sergej Sorli <sergej.sorli@arnes.si> in 2007

setlocale(LC_ALL, 'sl_SI.UTF-8');//1252 ACP UTF-8

//Parse csv file with graph data into Cacti database.
//Header file structre:
//<fqdn>;<subdescr>;<graphTemplateName>;<snmpQueryName>;<snmpQueryTypeName>;<snmpFieldName>;<snmpValueName>;<page>;<section>

/* do NOT run this script through a web browser */
if (!isset($_SERVER["argv"][0])){
	die("<br><strong>This script is only meant to run at the command line.</strong>");
}
$no_http_headers = true;

#################################################
#
# Well, usage
#
function display_help() {
	echo "Import Graphs SOAP Script 1.0, Copyright 2007 - The Cacti Group\n\n";
	echo "A simple command line SOAP utility to import graphs in Cacti\n\n";
	echo "usage: auto_add_graphs.php --username=[username] --password=[password]\n";
	echo "--urn=[urn] --source=[source file path] [--debug]\n";
}

#####################################################################
#
# Main
#
$debug = false; //default start

/* process calling arguments */
$parms = $_SERVER["argv"];
array_shift($parms);

	foreach($parms as $parameter) {
		@list($arg, $value) = @explode("=", $parameter);
		
		switch($arg)
		{
			case "--username":
				$username = trim($value);
				break;
			case "--password":
				$password = trim($value);
				break;
			case "--urn":
				$urn = trim($value);
				break;
			case "--source":
				$source = trim($value);
				break;
			case "--debug":
				$debug = true;
				break;
			default:
				display_help();
				exit(0);
		}
	}
	
	if ($username == "" || $password == ""){
		display_help();
		exit(1);
	}
	if ($urn == ""){
		display_help();
		exit(1);
	}
	if (!file_exists($source)){
		display_help();
		echo ("FAILED -- " . $source . " not exist\n"); 
		exit(1);
	}

//#############################################################

//Soap configuration
$soap_calls_retries = 3;
$url_Location = 'http://' . $urn . '/soap_service.php';
$urn_Location = 'urn:' . $urn . '';

$opts = array('location' => $url_Location, 'uri' => 'pc_SOAP_return_soap');
$client = new SOAPClient(null, $opts);
$auth = new SOAPAuth ($username, $password);
$header = new SOAPHeader($urn_Location, 'authenticate_user', $auth);
//#############################################################

//Source & destination files
$bckLOG = $source . "log.csv"; //bacup destination base
$errLOG = $source . "err.csv"; //errors
//#############################################################
//open files

if(!$debug)$errhandle = fopen($errLOG, "a");  //err records
if(!$debug)$bckhandle = fopen($bckLOG, "a");  //backup file

$handle = fopen($source, "r");
$row = 0;
while (($data = fgetcsv($handle, 1000, ";")) !== FALSE) {
	$row++; //$num = count($data);

	$retries = $soap_calls_retries;
	//Configuration fields
	$fqdn = $data[0];
	$graphTitle = $data[1];
	$graphTemplateName = $data[2];
	$snmpQueryName =$data[3];
	$snmpQueryTypeName = $data[4];
	$snmpFieldName = $data[5];
	$snmpValueName = $data[6];
	if (isset($data[7]))
	$tree_name = $data[7];
	else $tree_name = "";
	if (isset($data[8]))
	$node_name = $data[8];
	else $node_name = "";

	if($debug){
        echo "\nDetails:\n";
        echo "Row=" . $row . "\n";
        echo "fqdn=" . $fqdn . "\n";
        echo "graphTitle=" . $graphTitle . "\n";
        echo "graphTemplateName=" . $graphTemplateName . "\n";
        echo "snmpQueryName=" . $snmpQueryName . "\n";
        echo "snmpQueryTypeName=" . $snmpQueryTypeName . "\n";
        echo "snmpFieldName=" . $snmpFieldName . "\n";
        echo "snmpValueName=" . $snmpValueName . "\n";
        echo "tree_name=" . $tree_name . "\n";
        echo "node_name=" . $node_name . "\n";
	}
	//#############################################################
	do {
    	try {
        	$returnResult = $client->__soapCall('return_ID_get_Graph_Template_by_Name', array('graphTemplateName' => $graphTemplateName), array(), array($header));
        	if (!isset($returnResult["Errors"]) && empty($returnResult["Errors"])){
        		$template_id = $returnResult["Result"];
        
        		$returnResult = $client->__soapCall('return_ID_get_Host', array('hostName' => $fqdn), array(), array($header)); //Get host id in cacti database
        		if (!isset($returnResult["Errors"]) && empty($returnResult["Errors"])){
        			$hostId = $returnResult["Result"];
        
        			if (!($snmpValueName == "") && !empty($snmpQueryName) && !empty($snmpQueryTypeName) && !empty($snmpFieldName)){
        
        				$returnResult = $client->__soapCall('return_ID_get_SNMP_Query_By_Name', array('snmpQueryName' => $snmpQueryName), array(), array($header));
        				if (!isset($returnResult["Errors"]) && empty($returnResult["Errors"])){
        					$snmpQueryId = $returnResult["Result"];
        
        					$returnResult = $client->__soapCall('return_ID_get_SNMP_Query_Type_By_Name',
        					array('$snmpQueryTypeName' => $snmpQueryTypeName, 'snmpQueryId' => $snmpQueryId), array(), array($header));
        					if (!isset($returnResult["Errors"]) && empty($returnResult["Errors"])){
        						$snmpQueryTypeID = $returnResult["Result"];
        						
        						if($debug){
						        	echo "template_id=" . $template_id . "\n";
						        	echo "hostId=" . $hostId . "\n";
						        	echo "snmpQueryId=" . $snmpQueryId . "\n";
						        	echo "snmpQueryTypeID=" . $snmpQueryTypeID . "\n";
        						}
        						
        						$returnResult = $client->__soapCall('return_Name_get_SNMP_Field_By_Name', array('hostId' => $hostId, 'snmpFieldName' => $snmpFieldName), array(), array($header));
        						if (!isset($returnResult["Errors"]) && empty($returnResult["Errors"])){
        							$returnResult = $client->__soapCall('return_Field_get_SNMP_Value_by_Name', array('hostId' => $hostId, 'snmpFieldName' => $snmpFieldName, 'snmpValueName' => $snmpValueName), array(), array($header));
        							if (!isset($returnResult["Errors"]) && empty($returnResult["Errors"])){
        								
        								if(!$debug)$returnResult = $client->__soapCall('return_ID_add_Graph_DS',
        								array(  'graphTitle' => $graphTitle,
        								'template_id'=> $template_id,
        								'hostId' => $hostId,
        								'snmpQueryId' => $snmpQueryId,
        								'snmpQueryTypeID' => $snmpQueryTypeID,
        								'snmpFieldName' => $snmpFieldName,
        								'snmpValueName' => $snmpValueName
        								), array(), array($header));
        								//print error and resoult
                                    	if (!$debug)
                                    	if (!isset($returnResult["Errors"]) || empty($returnResult["Errors"])){
                                    		foreach($returnResult["Result"] as $graphID){
                                    			echo $row . ". Graph DS added - graph-id: (" . $graphID . ")\n";
                                    		}
                                    		if(!$debug)fputcsv($bckhandle, $data, ";");
                                    	}
        							}
        						}
        					}
        				}
        			}else{
        				if(!$debug)$returnResult = $client->__soapCall('return_ID_add_Graph_CG',
        				array(  'graphTitle' => $graphTitle,
        				'template_id'=> $template_id,
        				'hostId' => $hostId
        				), array(), array($header));
        				
                        if (!$debug)
                    	if (!isset($returnResult["Errors"]) || empty($returnResult["Errors"])){
                    	    foreach($returnResult["Result"] as $graphID){
                            	echo $row . ". Graph CG added - graph-id: (" . $graphID . ")\n";
                            }
                    		if(!$debug)fputcsv($bckhandle, $data, ";");
                    	}
        			}
        		}
        	}
        	$retries = 0;
    	}catch (Exception $e) {
        	echo ("\n" . $row . ". Message = " . $e->__toString() . "\n");
        	echo "Number of retries:" . $retries . "\n";
        	echo "Details:\n";
        	echo "fqdn=" . $fqdn . "\n";
        	echo "graphTitle=" . $graphTitle . "\n";
        	echo "graphTemplateName=" . $graphTemplateName . "\n";
        	echo "snmpQueryName=" . $snmpQueryName . "\n";
        	echo "snmpQueryTypeName=" . $snmpQueryTypeName . "\n";
        	echo "snmpFieldName=" . $snmpFieldName . "\n";
        	echo "snmpValueName=" . $snmpValueName . "\n";
        	echo "tree_name=" . $tree_name . "\n";
        	echo "node_name=" . $node_name . "\n";
        	
        	$retries--;
    	    if ($retries <= 0) {
        		$errors = array();        		
        		$catchException["value"] = ""; $catchException["variable"] = ""; $catchException["function"] = "SOAP call"; $catchException["description"] = $e->__toString();
            	array_push($errors, $catchException);
				$returnResult["Errors"] = $errors;
        	}
    	}
	} while ($retries > 0);
	
    if (isset($returnResult["Errors"]) && !empty($returnResult["Errors"])){
    	echo $row . ". Errors\n";
    	printThrowException($returnResult["Errors"]);
    	if(!$debug)fputcsv($errhandle, $data, ";");
    }
}

//close files
fclose($handle);    //close source csv file
if(!$debug)fclose($bckhandle); //backup log
if(!$debug)fclose($errhandle); //error log

//SOAP autentication
class SOAPAuth{
	public $username;
	public $password;

	public function __construct($username, $password){
		$this->username = $username;
		$this->password = $password;
	}
}

function printThrowException($errors){
	foreach($errors as $throwException){
		printf ("Error: " . $throwException["description"] . " - " . $throwException["function"] . "/" . $throwException["variable"] . "=" . $throwException["value"] . "\n");
	}
}
?>
